/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { ISequencedDocumentMessage } from "@fluidframework/protocol-definitions";
import { Heap, RedBlackTree } from "../collections";
import { ClientSeq } from "../mergeTree";
import { PropertySet } from "../properties";
import { TestClient } from "./testClient";
/**
 * Server for tests.  Simulates client communication by directing placing
 * messages in client queues.
 */
export declare class TestServer extends TestClient {
    seq: number;
    clients: TestClient[];
    private messageListeners;
    clientSeqNumbers: Heap<ClientSeq>;
    upstreamMap: RedBlackTree<number, number>;
    constructor(options?: PropertySet);
    addUpstreamClients(upstreamClients: TestClient[]): void;
    addClients(clients: TestClient[]): void;
    addListeners(listeners: TestClient[]): void;
    applyMsg(msg: ISequencedDocumentMessage): boolean;
    transformUpstreamMessage(msg: ISequencedDocumentMessage): void;
    copyMsg(msg: ISequencedDocumentMessage): ISequencedDocumentMessage;
    private minSeq;
    applyMessages(msgCount: number): boolean;
    incrementalGetText(start?: number, end?: number): string;
}
/**
 * Used for in-memory testing.  This will queue a reference string for each client message.
 */
export declare function checkTextMatchRelative(refSeq: number, clientId: number, server: TestServer, msg: ISequencedDocumentMessage): boolean;
//# sourceMappingURL=testServer.d.ts.map