/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { BaseSegment, ISegment } from "./mergeTreeNodes.mjs";
import { IJSONSegment } from "./ops.mjs";
import { PropertySet } from "./properties.mjs";
export declare const TextSegmentGranularity = 256;
/**
 * @internal
 */
export interface IJSONTextSegment extends IJSONSegment {
    text: string;
}
/**
 * @alpha
 */
export declare class TextSegment extends BaseSegment {
    text: string;
    static readonly type = "TextSegment";
    static is(segment: ISegment): segment is TextSegment;
    static make(text: string, props?: PropertySet): TextSegment;
    static fromJSONObject(spec: any): TextSegment | undefined;
    readonly type = "TextSegment";
    constructor(text: string);
    toJSONObject(): string | {
        text: string;
        props: PropertySet;
    };
    clone(start?: number, end?: number): TextSegment;
    canAppend(segment: ISegment): boolean;
    toString(): string;
    append(segment: ISegment): void;
    removeRange(start: number, end: number): boolean;
    protected createSplitSegmentAt(pos: number): TextSegment | undefined;
}
/**
 * @deprecated This functionality was not meant to be exported and will be removed in a future release
 * @alpha
 */
export interface IMergeTreeTextHelper {
    getText(refSeq: number, clientId: number, placeholder: string, start?: number, end?: number): string;
}
//# sourceMappingURL=textSegment.d.ts.map